@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-from __future__ import absolute_import, print_function, division
 import array
 import json as _json
 import os
@@ -17,10 +15,10 @@ from numcodecs.registry import get_codec
 from numcodecs import *  # noqa
 
 
-greetings = [u'¡Hola mundo!', u'Hej Världen!', u'Servus Woid!', u'Hei maailma!',
-             u'Xin chào thế giới', u'Njatjeta Botë!', u'Γεια σου κόσμε!',
-             u'こんにちは世界', u'世界，你好！', u'Helló, világ!', u'Zdravo svete!',
-             u'เฮลโลเวิลด์']
+greetings = ['¡Hola mundo!', 'Hej Världen!', 'Servus Woid!', 'Hei maailma!',
+             'Xin chào thế giới', 'Njatjeta Botë!', 'Γεια σου κόσμε!',
+             'こんにちは世界', '世界，你好！', 'Helló, világ!', 'Zdravo svete!',
+             'เฮลโลเวิลด์']
 
 
 def compare_arrays(arr, res, precision=None):
@@ -53,8 +51,7 @@ def check_encode_decode(arr, codec, precision=None):
     # we just require that the results of the encode/decode round-trip can
     # be compared to the original array.
 
-    # encoding should support any object exporting the buffer protocol,
-    # as well as array.array in PY2
+    # encoding should support any object exporting the buffer protocol
 
     # test encoding of numpy array
     enc = codec.encode(arr)
@@ -80,7 +77,6 @@ def check_encode_decode(arr, codec, precision=None):
     compare_arrays(arr, dec, precision=precision)
 
     # decoding should support any object exporting the buffer protocol,
-    # as well as array.array in PY2
 
     # setup
     enc_bytes = ensure_bytes(enc)
@@ -115,6 +111,75 @@ def check_encode_decode(arr, codec, precision=None):
     compare_arrays(arr, out, precision=precision)
 
 
+def check_encode_decode_partial(arr, codec, precision=None):
+
+    # N.B., watch out here with blosc compressor, if the itemsize of
+    # the source buffer is different then the results of encoding
+    # (i.e., compression) may be different. Hence we *do not* require that
+    # the results of encoding be identical for all possible inputs, rather
+    # we just require that the results of the encode/decode round-trip can
+    # be compared to the original array.
+
+    itemsize = arr.itemsize
+    start, nitems = 5, 10
+    compare_arr = arr[start:start+nitems]
+    # test encoding of numpy array
+    enc = codec.encode(arr)
+    dec = codec.decode_partial(enc, start, nitems)
+    compare_arrays(compare_arr, dec, precision=precision)
+
+    # out = np.empty_like(compare_arr)
+    out = np.empty_like(compare_arr)
+    print(len(out))
+
+    # test partial decode of encoded bytes
+    buf = arr.tobytes(order='A')
+    enc = codec.encode(buf)
+    dec = codec.decode_partial(enc, start*itemsize, nitems*itemsize, out=out)
+    compare_arrays(compare_arr, dec, precision=precision)
+
+    # test partial decode of encoded bytearray
+    buf = bytearray(arr.tobytes(order='A'))
+    enc = codec.encode(buf)
+    dec = codec.decode_partial(enc, start*itemsize, nitems*itemsize, out=out)
+    compare_arrays(compare_arr, dec, precision=precision)
+
+    # test partial decode of encoded array.array
+    buf = array.array('b', arr.tobytes(order='A'))
+    enc = codec.encode(buf)
+    dec = codec.decode_partial(enc, start*itemsize, nitems*itemsize, out=out)
+    compare_arrays(compare_arr, dec, precision=precision)
+
+    # # decoding should support any object exporting the buffer protocol,
+
+    # # setup
+    enc_bytes = ensure_bytes(enc)
+
+    # test decoding of raw bytes into numpy array
+    dec = codec.decode_partial(enc_bytes, start*itemsize, nitems*itemsize, out=out)
+    compare_arrays(compare_arr, dec, precision=precision)
+
+    # test partial decoding of bytearray
+    dec = codec.decode_partial(bytearray(enc_bytes), start*itemsize, nitems*itemsize, out=out)
+    compare_arrays(compare_arr, dec, precision=precision)
+
+    # test partial decoding of array.array
+    buf = array.array('b', enc_bytes)
+    dec = codec.decode_partial(buf, start*itemsize, nitems*itemsize, out=out)
+    compare_arrays(compare_arr, dec, precision=precision)
+
+    # test decoding of numpy array into numpy array
+    buf = np.frombuffer(enc_bytes, dtype='u1')
+    dec = codec.decode_partial(buf, start*itemsize, nitems*itemsize, out=out)
+    compare_arrays(compare_arr, dec, precision=precision)
+
+    # test decoding directly into bytearray
+    out = bytearray(compare_arr.nbytes)
+    codec.decode_partial(enc_bytes, start*itemsize, nitems*itemsize, out=out)
+    # noinspection PyTypeChecker
+    compare_arrays(compare_arr, out, precision=precision)
+
+
 def assert_array_items_equal(res, arr):
 
     assert isinstance(res, np.ndarray)
@@ -187,7 +252,7 @@ def check_backwards_compatibility(codec_id, arrays, codecs, precision=None, pref
 
         # setup
         i = int(arr_fn.split('.')[-2])
-        arr = np.load(arr_fn)
+        arr = np.load(arr_fn, allow_pickle=True)
         arr_bytes = arr.tobytes(order='A')
         if arr.flags.f_contiguous:
             order = 'F'
@@ -203,12 +268,10 @@ def check_backwards_compatibility(codec_id, arrays, codecs, precision=None, pref
 
             # file with codec configuration information
             codec_fn = os.path.join(codec_dir, 'config.json')
-
             # one time save config
             if not os.path.exists(codec_fn):  # pragma: no cover
                 with open(codec_fn, mode='w') as cf:
                     _json.dump(codec.get_config(), cf, sort_keys=True, indent=4)
-
             # load config and compare with expectation
             with open(codec_fn, mode='r') as cf:
                 config = _json.load(cf)
